@@ -19,11 +19,13 @@ import numpy as np
 from ..constants import G, g, me, omega, Rd, Re
 from ..package_tools import Exporter
 from ..units import atleast_1d, check_units, masked_array, units
+from ..xarray import preprocess_xarray
 
 exporter = Exporter(globals())
 
 
 @exporter.export
+@preprocess_xarray
 def get_wind_speed(u, v):
     r"""Compute the wind speed from u and v-components.
 
@@ -49,6 +51,7 @@ def get_wind_speed(u, v):
 
 
 @exporter.export
+@preprocess_xarray
 def get_wind_dir(u, v):
     r"""Compute the wind direction from u and v-components.
 
@@ -78,6 +81,7 @@ def get_wind_dir(u, v):
 
 
 @exporter.export
+@preprocess_xarray
 def get_wind_components(speed, wdir):
     r"""Calculate the U, V wind vector components from the speed and direction.
 
@@ -115,6 +119,7 @@ def get_wind_components(speed, wdir):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units(temperature='[temperature]', speed='[speed]')
 def windchill(temperature, speed, face_level_winds=False, mask_undefined=True):
     r"""Calculate the Wind Chill Temperature Index (WCTI).
@@ -176,6 +181,7 @@ def windchill(temperature, speed, face_level_winds=False, mask_undefined=True):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[temperature]')
 def heat_index(temperature, rh, mask_undefined=True):
     r"""Calculate the Heat Index from the current temperature and relative humidity.
@@ -230,6 +236,70 @@ def heat_index(temperature, rh, mask_undefined=True):
 
 
 @exporter.export
+@preprocess_xarray
+@check_units(temperature='[temperature]', speed='[speed]')
+def apparent_temperature(temperature, rh, speed, face_level_winds=False):
+    r"""Calculate the current apparent temperature.
+
+    Calculates the current apparent temperature based on the wind chill or heat index
+    as appropriate for the current conditions. Follows [NWS10201]_.
+
+    Parameters
+    ----------
+    temperature : `pint.Quantity`
+        The air temperature
+    rh : `pint.Quantity`
+        The relative humidity expressed as a unitless ratio in the range [0, 1].
+        Can also pass a percentage if proper units are attached.
+    speed : `pint.Quantity`
+        The wind speed at 10m.  If instead the winds are at face level,
+        `face_level_winds` should be set to `True` and the 1.5 multiplicative
+        correction will be applied automatically.
+    face_level_winds : bool, optional
+        A flag indicating whether the wind speeds were measured at facial
+        level instead of 10m, thus requiring a correction.  Defaults to
+        `False`.
+
+    Returns
+    -------
+    `pint.Quantity`
+        The corresponding apparent temperature value(s)
+
+    See Also
+    --------
+    heat_index, windchill
+
+    """
+    is_not_scalar = isinstance(temperature.m, (list, tuple, np.ndarray))
+
+    temperature = atleast_1d(temperature)
+    rh = atleast_1d(rh)
+    speed = atleast_1d(speed)
+
+    wind_chill_temperature = windchill(temperature, speed, face_level_winds=face_level_winds,
+                                       mask_undefined=True).to(temperature.units)
+
+    heat_index_temperature = heat_index(temperature, rh,
+                                        mask_undefined=True).to(temperature.units)
+
+    # Combine the heat index and wind chill arrays (no point has a value in both)
+    app_temperature = np.ma.where(wind_chill_temperature.mask,
+                                  heat_index_temperature,
+                                  wind_chill_temperature)
+
+    if is_not_scalar:
+        # Fill in missing areas where neither wind chill or heat index are applicable with the
+        # ambient temperature.
+        app_temperature[app_temperature.mask] = temperature[app_temperature.mask]
+        return np.array(app_temperature) * temperature.units
+    else:
+        if app_temperature.mask:
+            app_temperature = temperature.m
+        return app_temperature[0] * temperature.units
+
+
+@exporter.export
+@preprocess_xarray
 @check_units('[pressure]')
 def pressure_to_height_std(pressure):
     r"""Convert pressure data to heights using the U.S. standard atmosphere.
@@ -258,6 +328,7 @@ def pressure_to_height_std(pressure):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[length]')
 def height_to_geopotential(height):
     r"""Compute geopotential for a given height.
@@ -296,6 +367,7 @@ def height_to_geopotential(height):
 
 
 @exporter.export
+@preprocess_xarray
 def geopotential_to_height(geopot):
     r"""Compute height from a given geopotential.
 
@@ -337,6 +409,7 @@ def geopotential_to_height(geopot):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[length]')
 def height_to_pressure_std(height):
     r"""Convert height data to pressures using the U.S. standard atmosphere.
@@ -365,6 +438,7 @@ def height_to_pressure_std(height):
 
 
 @exporter.export
+@preprocess_xarray
 def coriolis_parameter(latitude):
     r"""Calculate the coriolis parameter at each point.
 
@@ -382,10 +456,11 @@ def coriolis_parameter(latitude):
 
     """
     latitude = _check_radians(latitude, max_radians=np.pi / 2)
-    return 2. * omega * np.sin(latitude)
+    return (2. * omega * np.sin(latitude)).to('1/s')
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[length]')
 def add_height_to_pressure(pressure, height):
     r"""Calculate the pressure at a certain height above another pressure level.
@@ -414,6 +489,7 @@ def add_height_to_pressure(pressure, height):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[length]', '[pressure]')
 def add_pressure_to_height(height, pressure):
     r"""Calculate the height at a certain pressure above another height.
@@ -442,6 +518,7 @@ def add_pressure_to_height(height, pressure):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[dimensionless]', '[pressure]', '[pressure]')
 def sigma_to_pressure(sigma, psfc, ptop):
     r"""Calculate pressure from sigma values.
