@@ -1,4 +1,4 @@
-# Copyright (c) 2009,2017 MetPy Developers.
+# Copyright (c) 2009,2017,2018 MetPy Developers.
 # Distributed under the terms of the BSD 3-Clause License.
 # SPDX-License-Identifier: BSD-3-Clause
 """Contains calculation of kinematic parameters (e.g. divergence or vorticity)."""
@@ -8,14 +8,15 @@ import functools
 import warnings
 
 import numpy as np
-from pyproj import Geod
 
-from ..calc.tools import first_derivative, get_layer_heights, gradient
+from . import coriolis_parameter
+from .tools import first_derivative, get_layer_heights, gradient
 from ..cbook import is_string_like, iterable
-from ..constants import Cp_d, g
+from ..constants import Cp_d, g, Rd
 from ..deprecation import deprecated
 from ..package_tools import Exporter
 from ..units import atleast_2d, check_units, concatenate, units
+from ..xarray import preprocess_xarray
 
 exporter = Exporter(globals())
 
@@ -90,6 +91,7 @@ def ensure_yx_order(func):
 
 
 @exporter.export
+@preprocess_xarray
 @ensure_yx_order
 def vorticity(u, v, dx, dy):
     r"""Calculate the vertical vorticity of the horizontal wind.
@@ -121,6 +123,7 @@ def vorticity(u, v, dx, dy):
 
 
 @exporter.export
+@preprocess_xarray
 @deprecated('0.7', addendum=' This function has been renamed vorticity.',
             pending=False)
 def v_vorticity(u, v, dx, dy, dim_order='xy'):
@@ -134,6 +137,7 @@ v_vorticity.__doc__ = (vorticity.__doc__ +
 
 
 @exporter.export
+@preprocess_xarray
 @ensure_yx_order
 def divergence(u, v, dx, dy):
     r"""Calculate the horizontal divergence of the horizontal wind.
@@ -165,6 +169,7 @@ def divergence(u, v, dx, dy):
 
 
 @exporter.export
+@preprocess_xarray
 @deprecated('0.7', addendum=' This function has been replaced by divergence.',
             pending=False)
 def h_convergence(u, v, dx, dy, dim_order='xy'):
@@ -178,6 +183,7 @@ h_convergence.__doc__ = (divergence.__doc__ +
 
 
 @exporter.export
+@preprocess_xarray
 @deprecated('0.7', addendum=' Use divergence and/or vorticity instead.',
             pending=False)
 @ensure_yx_order
@@ -218,6 +224,7 @@ def convergence_vorticity(u, v, dx, dy, dim_order='xy'):
 
 
 @exporter.export
+@preprocess_xarray
 @ensure_yx_order
 def shearing_deformation(u, v, dx, dy):
     r"""Calculate the shearing deformation of the horizontal wind.
@@ -249,6 +256,7 @@ def shearing_deformation(u, v, dx, dy):
 
 
 @exporter.export
+@preprocess_xarray
 @ensure_yx_order
 def stretching_deformation(u, v, dx, dy):
     r"""Calculate the stretching deformation of the horizontal wind.
@@ -280,6 +288,7 @@ def stretching_deformation(u, v, dx, dy):
 
 
 @exporter.export
+@preprocess_xarray
 @deprecated('0.7', addendum=' Use stretching_deformation and/or shearing_deformation instead.',
             pending=False)
 @ensure_yx_order
@@ -321,6 +330,7 @@ def shearing_stretching_deformation(u, v, dx, dy):
 
 
 @exporter.export
+@preprocess_xarray
 @ensure_yx_order
 def total_deformation(u, v, dx, dy):
     r"""Calculate the horizontal total deformation of the horizontal wind.
@@ -354,6 +364,7 @@ def total_deformation(u, v, dx, dy):
 
 
 @exporter.export
+@preprocess_xarray
 @ensure_yx_order
 def advection(scalar, wind, deltas):
     r"""Calculate the advection of a scalar field by the wind.
@@ -403,6 +414,7 @@ def advection(scalar, wind, deltas):
 
 
 @exporter.export
+@preprocess_xarray
 @ensure_yx_order
 def frontogenesis(thta, u, v, dx, dy, dim_order='yx'):
     r"""Calculate the 2D kinematic frontogenesis of a temperature field.
@@ -467,6 +479,7 @@ def frontogenesis(thta, u, v, dx, dy, dim_order='yx'):
 
 
 @exporter.export
+@preprocess_xarray
 @ensure_yx_order
 def geostrophic_wind(heights, f, dx, dy):
     r"""Calculate the geostrophic wind given from the heights or geopotential.
@@ -501,6 +514,42 @@ def geostrophic_wind(heights, f, dx, dy):
 
 
 @exporter.export
+@preprocess_xarray
+@ensure_yx_order
+def ageostrophic_wind(heights, f, dx, dy, u, v, dim_order='yx'):
+    r"""Calculate the ageostrophic wind given from the heights or geopotential.
+
+    Parameters
+    ----------
+    heights : (M, N) ndarray
+        The height field, with either leading dimensions of (x, y) or trailing dimensions
+        of (y, x), depending on the value of ``dim_order``.
+    f : array_like
+        The coriolis parameter.  This can be a scalar to be applied
+        everywhere or an array of values.
+    dx : scalar
+        The grid spacing in the x-direction
+    dy : scalar
+        The grid spacing in the y-direction
+    u : (M, N) ndarray
+        The u wind field, with either leading dimensions of (x, y) or trailing dimensions
+        of (y, x), depending on the value of ``dim_order``.
+    v : (M, N) ndarray
+        The u wind field, with either leading dimensions of (x, y) or trailing dimensions
+        of (y, x), depending on the value of ``dim_order``.
+
+    Returns
+    -------
+    A 2-item tuple of arrays
+        A tuple of the u-component and v-component of the ageostrophic wind.
+
+    """
+    u_geostrophic, v_geostrophic = geostrophic_wind(heights, f, dx, dy, dim_order=dim_order)
+    return u - u_geostrophic, v - v_geostrophic
+
+
+@exporter.export
+@preprocess_xarray
 @check_units('[length]', '[temperature]')
 def montgomery_streamfunction(height, temperature):
     r"""Compute the Montgomery Streamfunction on isentropic surfaces.
@@ -523,9 +572,11 @@ def montgomery_streamfunction(height, temperature):
 
     Notes
     -----
-    The formula used is that from [Lackmann2011]_ p. 69 for T in Kelvin and height in meters:
-    .. math:: sf = gZ + C_pT
-    * :math:`sf` is Montgomery Streamfunction
+    The formula used is that from [Lackmann2011]_ p. 69.
+
+    .. math:: \Psi = gZ + C_pT
+
+    * :math:`\Psi` is Montgomery Streamfunction
     * :math:`g` is avg. gravitational acceleration on Earth
     * :math:`Z` is geopotential height of the isentropic surface
     * :math:`C_p` is specific heat at constant pressure for dry air
@@ -540,6 +591,7 @@ def montgomery_streamfunction(height, temperature):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[speed]', '[speed]', '[length]', '[length]', '[length]',
              '[speed]', '[speed]')
 def storm_relative_helicity(u, v, heights, depth, bottom=0 * units.m,
@@ -595,7 +647,12 @@ def storm_relative_helicity(u, v, heights, depth, bottom=0 * units.m,
             (positive_srh + negative_srh).to('meter ** 2 / second ** 2'))
 
 
+@deprecated('0.8', addendum=' This function has been replaced by the signed delta distance'
+                            'calculation lat_lon_grid_deltas and will be removed in MetPy'
+                            ' 0.11.',
+            pending=False)
 @exporter.export
+@preprocess_xarray
 def lat_lon_grid_spacing(longitude, latitude, **kwargs):
     r"""Calculate the distance between grid points that are in a latitude/longitude format.
 
@@ -620,7 +677,46 @@ def lat_lon_grid_spacing(longitude, latitude, **kwargs):
     Accepts, 1D or 2D arrays for latitude and longitude
     Assumes [Y, X] for 2D arrays
 
+    .. deprecated:: 0.8.0
+        Function has been replaced with the signed delta distance calculation
+        `lat_lon_grid_deltas` and will be removed from MetPy in 0.11.0.
+
     """
+    # Use the absolute value of the signed function replacing this
+    dx, dy = lat_lon_grid_deltas(longitude, latitude, **kwargs)
+
+    return np.abs(dx), np.abs(dy)
+
+
+@exporter.export
+@preprocess_xarray
+def lat_lon_grid_deltas(longitude, latitude, **kwargs):
+    r"""Calculate the delta between grid points that are in a latitude/longitude format.
+
+    Calculate the signed delta distance between grid points when the grid spacing is defined by
+    delta lat/lon rather than delta x/y
+
+    Parameters
+    ----------
+    longitude : array_like
+        array of longitudes defining the grid
+    latitude : array_like
+        array of latitudes defining the grid
+    kwargs
+        Other keyword arguments to pass to :class:`~pyproj.Geod`
+
+    Returns
+    -------
+     dx, dy: 2D arrays of signed deltas between grid points in the x and y direction
+
+    Notes
+    -----
+    Accepts, 1D or 2D arrays for latitude and longitude
+    Assumes [Y, X] for 2D arrays
+
+    """
+    from pyproj import Geod
+
     # Inputs must be the same number of dimensions
     if latitude.ndim != longitude.ndim:
         raise ValueError('Latitude and longitude must have the same number of dimensions.')
@@ -630,10 +726,263 @@ def lat_lon_grid_spacing(longitude, latitude, **kwargs):
         longitude, latitude = np.meshgrid(longitude, latitude)
 
     geod_args = {'ellps': 'sphere'}
-    geod_args.update(**kwargs)
+    if kwargs:
+        geod_args = kwargs
+
     g = Geod(**geod_args)
 
-    _, _, dy = g.inv(longitude[:-1, :], latitude[:-1, :], longitude[1:, :], latitude[1:, :])
-    _, _, dx = g.inv(longitude[:, :-1], latitude[:, :-1], longitude[:, 1:], latitude[:, 1:])
+    forward_az, _, dy = g.inv(longitude[:-1, :], latitude[:-1, :], longitude[1:, :],
+                              latitude[1:, :])
+    dy[(forward_az < -90.) | (forward_az > 90.)] *= -1
+
+    forward_az, _, dx = g.inv(longitude[:, :-1], latitude[:, :-1], longitude[:, 1:],
+                              latitude[:, 1:])
+    dx[(forward_az < 0.) | (forward_az > 180.)] *= -1
 
     return dx * units.meter, dy * units.meter
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[speed]', '[speed]', '[length]', '[length]')
+def absolute_vorticity(u, v, dx, dy, lats, dim_order='yx'):
+    """Calculate the absolute vorticity of the horizontal wind.
+
+    Parameters
+    ----------
+    u : (M, N) ndarray
+        x component of the wind
+    v : (M, N) ndarray
+        y component of the wind
+    dx : float
+        The grid spacing in the x-direction
+    dy : float
+        The grid spacing in the y-direction
+    lats : (M, N) ndarray
+        latitudes of the wind data
+
+    Returns
+    -------
+    (M, N) ndarray
+        absolute vorticity
+
+    """
+    f = coriolis_parameter(lats)
+    relative_vorticity = vorticity(u, v, dx, dy, dim_order=dim_order)
+    return relative_vorticity + f
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[temperature]', '[pressure]', '[speed]', '[speed]',
+             '[length]', '[length]', '[dimensionless]')
+def potential_vorticity_baroclinic(potential_temperature, pressure, u, v, dx, dy, lats,
+                                   axis=0, dim_order='yx'):
+    r"""Calculate the baroclinic potential vorticity.
+
+    .. math:: PV = -g \frac{\partial \theta}{\partial z}(\zeta + f)
+
+    This formula is based on equation 7.31a [Hobbs2006]_.
+
+    Parameters
+    ----------
+    potential_temperature : (M, N, P) ndarray
+        potential temperature
+    pressure : (M, N, P) ndarray
+        vertical pressures
+    u : (M, N) ndarray
+        x component of the wind
+    v : (M, N) ndarray
+        y component of the wind
+    dx : float
+        The grid spacing in the x-direction
+    dy : float
+        The grid spacing in the y-direction
+    lats : (M, N) ndarray
+        latitudes of the wind data
+    axis : int, optional
+        The axis corresponding to the vertical dimension in the potential temperature
+        and pressure arrays, defaults to 0, the first dimension.
+
+    Returns
+    -------
+    (M, N) ndarray
+        baroclinic potential vorticity
+
+    Notes
+    -----
+    The same formula is used for isobaric and isentropic PV analysis. Provide winds
+    for vorticity calculations on the desired isobaric or isentropic surface. Three layers
+    of pressure/potential temperature are required in order to calculate the vertical
+    derivative (one above and below the desired surface).
+
+    """
+    if np.shape(potential_temperature)[axis] != 3:
+        raise ValueError('Length of potential temperature along axis '
+                         '{} must be 3.'.format(axis))
+    if np.shape(pressure)[axis] != 3:
+        raise ValueError('Length of pressure along axis '
+                         '{} must be 3.'.format(axis))
+    avor = absolute_vorticity(u, v, dx, dy, lats, dim_order=dim_order)
+    stability = first_derivative(potential_temperature, x=pressure, axis=axis)
+    # Get the middle layer stability derivative (index 1)
+    slices = [slice(None)] * stability.ndim
+    slices[axis] = 1
+    return (-1 * avor * g * stability[slices]).to(units.kelvin * units.meter**2 /
+                                                  (units.second * units.kilogram))
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[length]', '[speed]', '[speed]', '[length]', '[length]', '[dimensionless]')
+def potential_vorticity_barotropic(heights, u, v, dx, dy, lats, dim_order='yx'):
+    r"""Calculate the barotropic (Rossby) potential vorticity.
+
+    .. math:: PV = \frac{f + \zeta}{H}
+
+    This formula is based on equation 7.27 [Hobbs2006]_.
+
+    Parameters
+    ----------
+    heights : (M, N) ndarray
+        atmospheric heights
+    u : (M, N) ndarray
+        x component of the wind
+    v : (M, N) ndarray
+        y component of the wind
+    dx : float
+        The grid spacing in the x-direction
+    dy : float
+        The grid spacing in the y-direction
+    lats : (M, N) ndarray
+        latitudes of the wind data
+
+    Returns
+    -------
+    (M, N) ndarray
+        barotropic potential vorticity
+
+    """
+    avor = absolute_vorticity(u, v, dx, dy, lats, dim_order=dim_order)
+    return (avor / heights).to('meter**-1 * second**-1')
+
+
+@exporter.export
+@preprocess_xarray
+def inertial_advective_wind(u, v, u_geostrophic, v_geostrophic, dx, dy, lats):
+    r"""Calculate the inertial advective wind.
+
+    .. math:: \frac{\hat k}{f} \times (\vec V \cdot \nabla)\hat V_g
+
+    .. math:: \frac{\hat k}{f} \times \left[ \left( u \frac{\partial u_g}{\partial x} + v
+              \frac{\partial u_g}{\partial y} \right) \hat i + \left( u \frac{\partial v_g}
+              {\partial x} + v \frac{\partial v_g}{\partial y} \right) \hat j \right]
+
+    .. math:: \left[ -\frac{1}{f}\left(u \frac{\partial v_g}{\partial x} + v
+              \frac{\partial v_g}{\partial y} \right) \right] \hat i + \left[ \frac{1}{f}
+              \left( u \frac{\partial u_g}{\partial x} + v \frac{\partial u_g}{\partial y}
+              \right) \right] \hat j
+
+    This formula is based on equation 27 of [Rochette2006]_.
+
+    Parameters
+    ----------
+    u : (M, N) ndarray
+        x component of the advecting wind
+    v : (M, N) ndarray
+        y component of the advecting wind
+    u_geostrophic : (M, N) ndarray
+        x component of the geostrophic (advected) wind
+    v_geostrophic : (M, N) ndarray
+        y component of the geostrophic (advected) wind
+    dx : float
+        The grid spacing in the x-direction
+    dy : float
+        The grid spacing in the y-direction
+    lats : (M, N) ndarray
+        latitudes of the wind data
+
+    Returns
+    -------
+    (M, N) ndarray
+        x component of inertial advective wind
+    (M, N) ndarray
+        y component of inertial advective wind
+
+    Notes
+    -----
+    Many forms of the inertial advective wind assume the advecting and advected
+    wind to both be the geostrophic wind. To do so, pass the x and y components
+    of the geostrophic with for u and u_geostrophic/v and v_geostrophic.
+
+    """
+    f = coriolis_parameter(lats)
+
+    dugdx = first_derivative(u_geostrophic, delta=dx, axis=1)
+    dugdy = first_derivative(u_geostrophic, delta=dy, axis=0)
+    dvgdx = first_derivative(v_geostrophic, delta=dx, axis=1)
+    dvgdy = first_derivative(v_geostrophic, delta=dy, axis=0)
+
+    u_component = -(u * dvgdx + v * dvgdy) / f
+    v_component = (u * dugdx + v * dugdy) / f
+
+    return u_component, v_component
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[speed]', '[speed]', '[temperature]', '[pressure]', '[length]', '[length]')
+def q_vector(u, v, temperature, pressure, dx, dy, static_stability=1):
+    r"""Calculate Q-vector at a given pressure level using the u, v winds and temperature.
+
+    .. math:: \vec{Q} = (Q_1, Q_2)
+                      =  - \frac{R}{\sigma p}\left(
+                               \frac{\partial \vec{v}_g}{\partial x} \cdot \nabla_p T,
+                               \frac{\partial \vec{v}_g}{\partial y} \cdot \nabla_p T
+                           \right)
+
+    This formula follows equation 5.7.55 from [Bluestein1992]_, and can be used with the
+    the below form of the quasigeostrophic omega equation to assess vertical motion
+    ([Bluestein1992]_ equation 5.7.54):
+
+    .. math:: \left( \nabla_p^2 + \frac{f_0^2}{\sigma} \frac{\partial^2}{\partial p^2}
+                  \right) \omega =
+              - 2 \nabla_p \cdot \vec{Q} -
+                  \frac{R}{\sigma p} \beta \frac{\partial T}{\partial x}.
+
+    Parameters
+    ----------
+    u : (M, N) ndarray
+        x component of the wind (geostrophic in QG-theory)
+    v : (M, N) ndarray
+        y component of the wind (geostrophic in QG-theory)
+    temperature : (M, N) ndarray
+        Array of temperature at pressure level
+    pressure : `pint.Quantity`
+        Pressure at level
+    dx : float
+        The grid spacing in the x-direction
+    dy : float
+        The grid spacing in the y-direction
+    static_stability : `pint.Quantity`, optional
+        The static stability at the pressure level. Defaults to 1 if not given to calculate
+        the Q-vector without factoring in static stability.
+
+    Returns
+    -------
+    tuple of (M, N) ndarrays
+        The components of the Q-vector in the u- and v-directions respectively
+
+    See Also
+    --------
+    static_stability
+
+    """
+    dudy, dudx = gradient(u, deltas=(dy, dx), axis=(-2, -1))
+    dvdy, dvdx = gradient(v, deltas=(dy, dx), axis=(-2, -1))
+    dtempdy, dtempdx = gradient(temperature, deltas=(dy, dx), axis=(-2, -1))
+
+    q1 = -Rd / (pressure * static_stability) * (dudx * dtempdx + dvdx * dtempdy)
+    q2 = -Rd / (pressure * static_stability) * (dudy * dtempdx + dvdy * dtempdy)
+
+    return q1.to_base_units(), q2.to_base_units()
