@@ -1,4 +1,4 @@
-# Copyright (c) 2008,2015,2016,2017 MetPy Developers.
+# Copyright (c) 2008,2015,2016,2017,2018 MetPy Developers.
 # Distributed under the terms of the BSD 3-Clause License.
 # SPDX-License-Identifier: BSD-3-Clause
 """Contains a collection of thermodynamic calculations."""
@@ -11,11 +11,13 @@ import numpy as np
 import scipy.integrate as si
 import scipy.optimize as so
 
-from .tools import (_greater_or_close, _less_or_close, broadcast_indices, find_intersections,
-                    get_layer, interp)
+from .tools import (_greater_or_close, _less_or_close, broadcast_indices,
+                    find_bounding_indices, find_intersections, first_derivative, get_layer,
+                    interp)
 from ..constants import Cp_d, epsilon, g, kappa, Lv, P0, Rd
 from ..package_tools import Exporter
 from ..units import atleast_1d, check_units, concatenate, units
+from ..xarray import preprocess_xarray
 
 exporter = Exporter(globals())
 
@@ -23,6 +25,7 @@ sat_pressure_0c = 6.112 * units.millibar
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[temperature]', '[temperature]')
 def relative_humidity_from_dewpoint(temperature, dewpt):
     r"""Calculate the relative humidity.
@@ -53,6 +56,41 @@ def relative_humidity_from_dewpoint(temperature, dewpt):
 
 
 @exporter.export
+@preprocess_xarray
+@check_units('[pressure]', '[pressure]')
+def exner_function(pressure, reference_pressure=P0):
+    r"""Calculate the Exner function.
+
+    .. math:: \Pi = \left( \frac{p}{p_0} \right)^\kappa
+
+    This can be used to calculate potential temperature from temperature (and visa-versa),
+    since
+
+    .. math:: \Pi = \frac{T}{\theta}
+
+    Parameters
+    ----------
+    pressure : `pint.Quantity`
+        The total atmospheric pressure
+    reference_pressure : `pint.Quantity`, optional
+        The reference pressure against which to calculate the Exner function, defaults to P0
+
+    Returns
+    -------
+    `pint.Quantity`
+        The value of the Exner function at the given pressure
+
+    See Also
+    --------
+    potential_temperature
+    temperature_from_potential_temperature
+
+    """
+    return (pressure / reference_pressure).to('dimensionless')**kappa
+
+
+@exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]')
 def potential_temperature(pressure, temperature):
     r"""Calculate the potential temperature.
@@ -90,10 +128,56 @@ def potential_temperature(pressure, temperature):
     <Quantity(290.96653180346203, 'kelvin')>
 
     """
-    return temperature * (P0 / pressure).to('dimensionless')**kappa
+    return temperature / exner_function(pressure)
 
 
 @exporter.export
+@preprocess_xarray
+@check_units('[pressure]', '[temperature]')
+def temperature_from_potential_temperature(pressure, theta):
+    r"""Calculate the temperature from a given potential temperature.
+
+    Uses the inverse of the Poisson equation to calculate the temperature from a
+    given potential temperature at a specific pressure level.
+
+    Parameters
+    ----------
+    pressure : `pint.Quantity`
+        The total atmospheric pressure
+    theta : `pint.Quantity`
+        The potential temperature
+
+    Returns
+    -------
+    `pint.Quantity`
+        The temperature corresponding to the potential temperature and pressure.
+
+    See Also
+    --------
+    dry_lapse
+    potential_temperature
+
+    Notes
+    -----
+    Formula:
+
+    .. math:: T = \Theta (P / P_0)^\kappa
+
+    Examples
+    --------
+    >>> from metpy.units import units
+    >>> from metpy.calc import temperature_from_potential_temperature
+    >>> # potential temperature
+    >>> theta = np.array([ 286.12859679, 288.22362587]) * units.kelvin
+    >>> p = 850 * units.mbar
+    >>> T = temperature_from_potential_temperature(p,theta)
+
+    """
+    return theta * exner_function(pressure)
+
+
+@exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]')
 def dry_lapse(pressure, temperature):
     r"""Calculate the temperature at a level assuming only dry processes.
@@ -126,6 +210,7 @@ def dry_lapse(pressure, temperature):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]')
 def moist_lapse(pressure, temperature):
     r"""Calculate the temperature at a level assuming liquid saturation processes.
@@ -175,6 +260,7 @@ def moist_lapse(pressure, temperature):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]', '[temperature]')
 def lcl(pressure, temperature, dewpt, max_iters=50, eps=1e-5):
     r"""Calculate the lifted condensation level (LCL) using from the starting point.
@@ -231,8 +317,9 @@ def lcl(pressure, temperature, dewpt, max_iters=50, eps=1e-5):
 
 
 @exporter.export
-@check_units('[pressure]', '[temperature]', '[temperature]')
-def lfc(pressure, temperature, dewpt):
+@preprocess_xarray
+@check_units('[pressure]', '[temperature]', '[temperature]', '[temperature]')
+def lfc(pressure, temperature, dewpt, parcel_temperature_profile=None):
     r"""Calculate the level of free convection (LFC).
 
     This works by finding the first intersection of the ideal parcel path and
@@ -246,6 +333,9 @@ def lfc(pressure, temperature, dewpt):
         The temperature at the levels given by `pressure`
     dewpt : `pint.Quantity`
         The dew point at the levels given by `pressure`
+    parcel_temperature_profile: `pint.Quantity`, optional
+        The parcel temperature profile from which to calculate the LFC. Defaults to the
+        surface parcel profile.
 
     Returns
     -------
@@ -257,27 +347,41 @@ def lfc(pressure, temperature, dewpt):
     parcel_profile
 
     """
-    ideal_profile = parcel_profile(pressure, temperature[0], dewpt[0]).to('degC')
-
+    # Default to surface parcel if no profile or starting pressure level is given
+    if parcel_temperature_profile is None:
+        parcel_temperature_profile = parcel_profile(pressure, temperature[0],
+                                                    dewpt[0]).to('degC')
     # The parcel profile and data have the same first data point, so we ignore
     # that point to get the real first intersection for the LFC calculation.
-    x, y = find_intersections(pressure[1:], ideal_profile[1:], temperature[1:],
-                              direction='increasing')
-    # Two possible cases here: LFC = LCL, or LFC doesn't exist
+    x, y = find_intersections(pressure[1:], parcel_temperature_profile[1:],
+                              temperature[1:], direction='increasing')
+
+    # The LFC could:
+    # 1) Not exist
+    # 2) Exist but be equal to the LCL
+    # 3) Exist and be above the LCL
+
+    # LFC does not exist or is LCL
     if len(x) == 0:
-        if np.all(_less_or_close(ideal_profile, temperature)):
+        if np.all(_less_or_close(parcel_temperature_profile, temperature)):
             # LFC doesn't exist
             return np.nan * pressure.units, np.nan * temperature.units
         else:  # LFC = LCL
             x, y = lcl(pressure[0], temperature[0], dewpt[0])
             return x, y
+
+    # LFC exists and is not LCL. Make sure it is above the LCL.
     else:
+        idx = x < lcl(pressure[0], temperature[0], dewpt[0])[0]
+        x = x[idx]
+        y = y[idx]
         return x[0], y[0]
 
 
 @exporter.export
-@check_units('[pressure]', '[temperature]', '[temperature]')
-def el(pressure, temperature, dewpt):
+@preprocess_xarray
+@check_units('[pressure]', '[temperature]', '[temperature]', '[temperature]')
+def el(pressure, temperature, dewpt, parcel_temperature_profile=None):
     r"""Calculate the equilibrium level.
 
     This works by finding the last intersection of the ideal parcel path and
@@ -292,6 +396,9 @@ def el(pressure, temperature, dewpt):
         The temperature at the levels given by `pressure`
     dewpt : `pint.Quantity`
         The dew point at the levels given by `pressure`
+    parcel_temperature_profile: `pint.Quantity`, optional
+        The parcel temperature profile from which to calculate the EL. Defaults to the
+        surface parcel profile.
 
     Returns
     -------
@@ -303,13 +410,16 @@ def el(pressure, temperature, dewpt):
     parcel_profile
 
     """
-    ideal_profile = parcel_profile(pressure, temperature[0], dewpt[0]).to('degC')
+    # Default to surface parcel if no profile or starting pressure level is given
+    if parcel_temperature_profile is None:
+        parcel_temperature_profile = parcel_profile(pressure, temperature[0],
+                                                    dewpt[0]).to('degC')
     # If the top of the sounding parcel is warmer than the environment, there is no EL
-    if ideal_profile[-1] > temperature[-1]:
+    if parcel_temperature_profile[-1] > temperature[-1]:
         return np.nan * pressure.units, np.nan * temperature.units
 
     # Otherwise the last intersection (as long as there is one) is the EL
-    x, y = find_intersections(pressure[1:], ideal_profile[1:], temperature[1:])
+    x, y = find_intersections(pressure[1:], parcel_temperature_profile[1:], temperature[1:])
     if len(x) > 0:
         return x[-1], y[-1]
     else:
@@ -317,6 +427,7 @@ def el(pressure, temperature, dewpt):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]', '[temperature]')
 def parcel_profile(pressure, temperature, dewpt):
     r"""Calculate the profile a parcel takes through the atmosphere.
@@ -346,7 +457,8 @@ def parcel_profile(pressure, temperature, dewpt):
 
     """
     # Find the LCL
-    lcl_pressure = lcl(pressure[0], temperature, dewpt)[0].to(pressure.units)
+    lcl_pressure, _ = lcl(pressure[0], temperature, dewpt)
+    lcl_pressure = lcl_pressure.to(pressure.units)
 
     # Find the dry adiabatic profile, *including* the LCL. We need >= the LCL in case the
     # LCL is included in the levels. It's slightly redundant in that case, but simplifies
@@ -354,6 +466,10 @@ def parcel_profile(pressure, temperature, dewpt):
     press_lower = concatenate((pressure[pressure >= lcl_pressure], lcl_pressure))
     t1 = dry_lapse(press_lower, temperature)
 
+    # If the pressure profile doesn't make it to the lcl, we can stop here
+    if _greater_or_close(np.nanmin(pressure), lcl_pressure.m):
+        return t1[:-1]
+
     # Find moist pseudo-adiabatic profile starting at the LCL
     press_upper = concatenate((lcl_pressure, pressure[pressure < lcl_pressure]))
     t2 = moist_lapse(press_upper, t1[-1]).to(t1.units)
@@ -363,6 +479,7 @@ def parcel_profile(pressure, temperature, dewpt):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[dimensionless]')
 def vapor_pressure(pressure, mixing):
     r"""Calculate water vapor (partial) pressure.
@@ -399,6 +516,7 @@ def vapor_pressure(pressure, mixing):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[temperature]')
 def saturation_vapor_pressure(temperature):
     r"""Calculate the saturation water vapor (partial) pressure.
@@ -434,6 +552,7 @@ def saturation_vapor_pressure(temperature):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[temperature]', '[dimensionless]')
 def dewpoint_rh(temperature, rh):
     r"""Calculate the ambient dewpoint given air temperature and relative humidity.
@@ -443,7 +562,7 @@ def dewpoint_rh(temperature, rh):
     temperature : `pint.Quantity`
         Air temperature
     rh : `pint.Quantity`
-        Relative humidity expressed as a ratio in the range (0, 1]
+        Relative humidity expressed as a ratio in the range 0 < rh <= 1
 
     Returns
     -------
@@ -461,6 +580,7 @@ def dewpoint_rh(temperature, rh):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]')
 def dewpoint(e):
     r"""Calculate the ambient dewpoint given the vapor pressure.
@@ -493,6 +613,7 @@ def dewpoint(e):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[pressure]', '[dimensionless]')
 def mixing_ratio(part_press, tot_press, molecular_weight_ratio=epsilon):
     r"""Calculate the mixing ratio of a gas.
@@ -534,6 +655,7 @@ def mixing_ratio(part_press, tot_press, molecular_weight_ratio=epsilon):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]')
 def saturation_mixing_ratio(tot_press, temperature):
     r"""Calculate the saturation mixing ratio of water vapor.
@@ -558,6 +680,7 @@ def saturation_mixing_ratio(tot_press, temperature):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]', '[temperature]')
 def equivalent_potential_temperature(pressure, temperature, dewpoint):
     r"""Calculate equivalent potential temperature.
@@ -614,6 +737,74 @@ def equivalent_potential_temperature(pressure, temperature, dewpoint):
 
 
 @exporter.export
+@preprocess_xarray
+@check_units('[pressure]', '[temperature]')
+def saturation_equivalent_potential_temperature(pressure, temperature):
+    r"""Calculate saturation equivalent potential temperature.
+
+    This calculation must be given an air parcel's pressure and temperature.
+    The implementation uses the formula outlined in [Bolton1980]_ for the
+    equivalent potential temperature, and assumes a saturated process.
+
+    First, because we assume a saturated process, the temperature at the LCL is
+    equivalent to the current temperature. Therefore the following equation
+
+    .. math:: T_{L}=\frac{1}{\frac{1}{T_{D}-56}+\frac{ln(T_{K}/T_{D})}{800}}+56
+
+    reduces to
+
+    .. math:: T_{L} = T_{K}
+
+    Then the potential temperature at the temperature/LCL is calculated:
+
+    .. math:: \theta_{DL}=T_{K}\left(\frac{1000}{p-e}\right)^k
+              \left(\frac{T_{K}}{T_{L}}\right)^{.28r}
+
+    However, because
+
+    .. math:: T_{L} = T_{K}
+
+    it follows that
+
+    .. math:: \theta_{DL}=T_{K}\left(\frac{1000}{p-e}\right)^k
+
+    Both of these are used to calculate the final equivalent potential temperature:
+
+    .. math:: \theta_{E}=\theta_{DL}\exp\left[\left(\frac{3036.}{T_{K}}
+                                              -1.78\right)*r(1+.448r)\right]
+
+    Parameters
+    ----------
+    pressure: `pint.Quantity`
+        Total atmospheric pressure
+    temperature: `pint.Quantity`
+        Temperature of parcel
+
+    Returns
+    -------
+    `pint.Quantity`
+        The saturation equivalent potential temperature of the parcel
+
+    Notes
+    -----
+    [Bolton1980]_ formula for Theta-e is used (for saturated case), since according to
+    [DaviesJones2009]_ it is the most accurate non-iterative formulation
+    available.
+
+    """
+    t = temperature.to('kelvin').magnitude
+    p = pressure.to('hPa').magnitude
+    e = saturation_vapor_pressure(temperature).to('hPa').magnitude
+    r = saturation_mixing_ratio(pressure, temperature).magnitude
+
+    th_l = t * (1000 / (p - e)) ** kappa
+    th_es = th_l * np.exp((3036. / t - 1.78) * r * (1 + 0.448 * r))
+
+    return th_es * units.kelvin
+
+
+@exporter.export
+@preprocess_xarray
 @check_units('[temperature]', '[dimensionless]', '[dimensionless]')
 def virtual_temperature(temperature, mixing, molecular_weight_ratio=epsilon):
     r"""Calculate virtual temperature.
@@ -647,6 +838,7 @@ def virtual_temperature(temperature, mixing, molecular_weight_ratio=epsilon):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]', '[dimensionless]', '[dimensionless]')
 def virtual_potential_temperature(pressure, temperature, mixing,
                                   molecular_weight_ratio=epsilon):
@@ -683,6 +875,7 @@ def virtual_potential_temperature(pressure, temperature, mixing,
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]', '[dimensionless]', '[dimensionless]')
 def density(pressure, temperature, mixing, molecular_weight_ratio=epsilon):
     r"""Calculate density.
@@ -718,6 +911,7 @@ def density(pressure, temperature, mixing, molecular_weight_ratio=epsilon):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[temperature]', '[temperature]', '[pressure]')
 def relative_humidity_wet_psychrometric(dry_bulb_temperature, web_bulb_temperature,
                                         pressure, **kwargs):
@@ -759,6 +953,7 @@ def relative_humidity_wet_psychrometric(dry_bulb_temperature, web_bulb_temperatu
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[temperature]', '[temperature]', '[pressure]')
 def psychrometric_vapor_pressure_wet(dry_bulb_temperature, wet_bulb_temperature, pressure,
                                      psychrometer_coefficient=6.21e-4 / units.kelvin):
@@ -808,6 +1003,7 @@ def psychrometric_vapor_pressure_wet(dry_bulb_temperature, wet_bulb_temperature,
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[dimensionless]', '[temperature]', '[pressure]')
 def mixing_ratio_from_relative_humidity(relative_humidity, temperature, pressure):
     r"""Calculate the mixing ratio from relative humidity, temperature, and pressure.
@@ -842,10 +1038,12 @@ def mixing_ratio_from_relative_humidity(relative_humidity, temperature, pressure
     relative_humidity_from_mixing_ratio, saturation_mixing_ratio
 
     """
-    return relative_humidity * saturation_mixing_ratio(pressure, temperature)
+    return (relative_humidity *
+            saturation_mixing_ratio(pressure, temperature)).to('dimensionless')
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[dimensionless]', '[temperature]', '[pressure]')
 def relative_humidity_from_mixing_ratio(mixing_ratio, temperature, pressure):
     r"""Calculate the relative humidity from mixing ratio, temperature, and pressure.
@@ -883,6 +1081,7 @@ def relative_humidity_from_mixing_ratio(mixing_ratio, temperature, pressure):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[dimensionless]')
 def mixing_ratio_from_specific_humidity(specific_humidity):
     r"""Calculate the mixing ratio from specific humidity.
@@ -919,6 +1118,7 @@ def mixing_ratio_from_specific_humidity(specific_humidity):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[dimensionless]')
 def specific_humidity_from_mixing_ratio(mixing_ratio):
     r"""Calculate the specific humidity from the mixing ratio.
@@ -955,6 +1155,7 @@ def specific_humidity_from_mixing_ratio(mixing_ratio):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[dimensionless]', '[temperature]', '[pressure]')
 def relative_humidity_from_specific_humidity(specific_humidity, temperature, pressure):
     r"""Calculate the relative humidity from specific humidity, temperature, and pressure.
@@ -993,6 +1194,7 @@ def relative_humidity_from_specific_humidity(specific_humidity, temperature, pre
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]', '[temperature]', '[temperature]')
 def cape_cin(pressure, temperature, dewpt, parcel_profile):
     r"""Calculate CAPE and CIN.
@@ -1047,7 +1249,8 @@ def cape_cin(pressure, temperature, dewpt, parcel_profile):
 
     """
     # Calculate LFC limit of integration
-    lfc_pressure = lfc(pressure, temperature, dewpt)[0]
+    lfc_pressure, _ = lfc(pressure, temperature, dewpt,
+                          parcel_temperature_profile=parcel_profile)
 
     # If there is no LFC, no need to proceed.
     if np.isnan(lfc_pressure):
@@ -1056,7 +1259,8 @@ def cape_cin(pressure, temperature, dewpt, parcel_profile):
         lfc_pressure = lfc_pressure.magnitude
 
     # Calculate the EL limit of integration
-    el_pressure = el(pressure, temperature, dewpt)[0]
+    el_pressure, _ = el(pressure, temperature, dewpt,
+                        parcel_temperature_profile=parcel_profile)
 
     # No EL and we use the top reading of the sounding.
     if np.isnan(el_pressure):
@@ -1127,6 +1331,7 @@ def _find_append_zero_crossings(x, y):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]', '[temperature]')
 def most_unstable_parcel(pressure, temperature, dewpoint, heights=None,
                          bottom=None, depth=300 * units.hPa):
@@ -1165,14 +1370,15 @@ def most_unstable_parcel(pressure, temperature, dewpoint, heights=None,
     get_layer
 
     """
-    p_layer, T_layer, Td_layer = get_layer(pressure, temperature, dewpoint, bottom=bottom,
+    p_layer, t_layer, td_layer = get_layer(pressure, temperature, dewpoint, bottom=bottom,
                                            depth=depth, heights=heights, interpolate=False)
-    theta_e = equivalent_potential_temperature(p_layer, T_layer, Td_layer)
+    theta_e = equivalent_potential_temperature(p_layer, t_layer, td_layer)
     max_idx = np.argmax(theta_e)
-    return p_layer[max_idx], T_layer[max_idx], Td_layer[max_idx], max_idx
+    return p_layer[max_idx], t_layer[max_idx], td_layer[max_idx], max_idx
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[temperature]', '[pressure]', '[temperature]')
 def isentropic_interpolation(theta_levels, pressure, temperature, *args, **kwargs):
     r"""Interpolate data in isobaric coordinates to isentropic coordinates.
@@ -1205,6 +1411,9 @@ def isentropic_interpolation(theta_levels, pressure, temperature, *args, **kwarg
         The maximum number of iterations to use in calculation, defaults to 50.
     eps : float, optional
         The desired absolute error in the calculated value, defaults to 1e-6.
+    bottom_up_search : bool, optional
+        Controls whether to search for theta levels bottom-up, or top-down. Defaults to
+        True, which is bottom-up search.
 
     Notes
     -----
@@ -1236,6 +1445,7 @@ def isentropic_interpolation(theta_levels, pressure, temperature, *args, **kwarg
     max_iters = kwargs.pop('max_iters', 50)
     eps = kwargs.pop('eps', 1e-6)
     axis = kwargs.pop('axis', 0)
+    bottom_up_search = kwargs.pop('bottom_up_search', True)
 
     # Get dimensions in temperature
     ndim = temperature.ndim
@@ -1246,28 +1456,25 @@ def isentropic_interpolation(theta_levels, pressure, temperature, *args, **kwarg
 
     slices = [np.newaxis] * ndim
     slices[axis] = slice(None)
-    pres = pres[slices]
-    pres = np.broadcast_to(pres, temperature.shape) * pres.units
+    pres = np.broadcast_to(pres[slices], temperature.shape) * pres.units
 
     # Sort input data
     sort_pres = np.argsort(pres.m, axis=axis)
     sort_pres = np.swapaxes(np.swapaxes(sort_pres, 0, axis)[::-1], 0, axis)
     sorter = broadcast_indices(pres, sort_pres, ndim, axis)
     levs = pres[sorter]
-    theta_levels = np.asanyarray(theta_levels.to('kelvin')).reshape(-1)
-    sort_isentlevs = np.argsort(theta_levels)
     tmpk = temperature[sorter]
-    isentlevels = theta_levels[sort_isentlevs]
+
+    theta_levels = np.asanyarray(theta_levels.to('kelvin')).reshape(-1)
+    isentlevels = theta_levels[np.argsort(theta_levels)]
 
     # Make the desired isentropic levels the same shape as temperature
-    isentlevs_nd = isentlevels
-    isentlevs_nd = isentlevs_nd[slices]
     shape = list(temperature.shape)
     shape[axis] = isentlevels.size
-    isentlevs_nd = np.broadcast_to(isentlevs_nd, shape)
+    isentlevs_nd = np.broadcast_to(isentlevels[slices], shape)
 
     # exponent to Poisson's Equation, which is imported above
-    ka = kappa.to('dimensionless').m
+    ka = kappa.m_as('dimensionless')
 
     # calculate theta for each point
     pres_theta = potential_temperature(levs, tmpk)
@@ -1284,55 +1491,51 @@ def isentropic_interpolation(theta_levels, pressure, temperature, *args, **kwarg
     pok = P0 ** ka
 
     # index values for each point for the pressure level nearest to the desired theta level
-    minv = np.apply_along_axis(np.searchsorted, axis, pres_theta.m, theta_levels)
-
-    # Create index values for broadcasting arrays
-    above = broadcast_indices(tmpk, minv, ndim, axis)
-    below = broadcast_indices(tmpk, minv - 1, ndim, axis)
+    above, below, good = find_bounding_indices(pres_theta.m, theta_levels, axis,
+                                               from_below=bottom_up_search)
 
     # calculate constants for the interpolation
     a = (tmpk.m[above] - tmpk.m[below]) / (log_p[above] - log_p[below])
     b = tmpk.m[above] - a * log_p[above]
 
     # calculate first guess for interpolation
-    first_guess = 0.5 * (log_p[above] + log_p[below])
+    isentprs = 0.5 * (log_p[above] + log_p[below])
+
+    # Make sure we ignore any nans in the data for solving; checking a is enough since it
+    # combines log_p and tmpk.
+    good &= ~np.isnan(a)
 
     # iterative interpolation using scipy.optimize.fixed_point and _isen_iter defined above
-    log_p_solved = so.fixed_point(_isen_iter, first_guess, args=(isentlevs_nd, ka, a, b,
-                                                                 pok.m),
+    log_p_solved = so.fixed_point(_isen_iter, isentprs[good],
+                                  args=(isentlevs_nd[good], ka, a[good], b[good], pok.m),
                                   xtol=eps, maxiter=max_iters)
 
-    # get back pressure and assign nan for values with pressure greater than 1000 hPa
-    isentprs = np.exp(log_p_solved)
-    isentprs[isentprs > np.max(pressure.m)] = np.nan
+    # get back pressure from log p
+    isentprs[good] = np.exp(log_p_solved)
+
+    # Mask out points we know are bad as well as points that are beyond the max pressure
+    isentprs[~(good & _less_or_close(isentprs, np.max(pres.m)))] = np.nan
 
     # create list for storing output data
-    ret = []
-    ret.append(isentprs * units.hPa)
+    ret = [isentprs * units.hPa]
 
     # if tmpk_out = true, calculate temperature and output as last item in list
     if tmpk_out:
         ret.append((isentlevs_nd / ((P0.m / isentprs) ** ka)) * units.kelvin)
 
-    # check to see if any additional arguments were given, if so, interpolate to
-    # new isentropic levels
-    try:
-        args[0]
-    except IndexError:
-        return ret
-    else:
-        # do an interpolation for each additional argument
-        for arr in args:
-            var = arr[sorter]
-            # interpolate to isentropic levels and add to temporary output array
-            arg_out = interp(isentlevels, pres_theta.m, var, axis=axis)
-            ret.append(arg_out)
+    # do an interpolation for each additional argument
+    if args:
+        others = interp(isentlevels, pres_theta.m, *(arr[sorter] for arr in args), axis=axis)
+        if len(args) > 1:
+            ret.extend(others)
+        else:
+            ret.append(others)
 
-    # output values as a list
     return ret
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]', '[temperature]')
 def surface_based_cape_cin(pressure, temperature, dewpoint):
     r"""Calculate surface-based CAPE and CIN.
@@ -1370,6 +1573,7 @@ def surface_based_cape_cin(pressure, temperature, dewpoint):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]', '[temperature]')
 def most_unstable_cape_cin(pressure, temperature, dewpoint, **kwargs):
     r"""Calculate most unstable CAPE/CIN.
@@ -1411,6 +1615,7 @@ def most_unstable_cape_cin(pressure, temperature, dewpoint, **kwargs):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]', '[temperature]')
 def mixed_parcel(p, temperature, dewpt, parcel_start_pressure=None,
                  heights=None, bottom=None, depth=100 * units.hPa, interpolate=True):
@@ -1472,6 +1677,7 @@ def mixed_parcel(p, temperature, dewpt, parcel_start_pressure=None,
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]')
 def mixed_layer(p, *args, **kwargs):
     r"""Mix variable(s) over a layer, yielding a mass-weighted average.
@@ -1522,6 +1728,7 @@ def mixed_layer(p, *args, **kwargs):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[length]', '[temperature]')
 def dry_static_energy(heights, temperature):
     r"""Calculate the dry static energy of parcels.
@@ -1553,6 +1760,7 @@ def dry_static_energy(heights, temperature):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[length]', '[temperature]', '[dimensionless]')
 def moist_static_energy(heights, temperature, specific_humidity):
     r"""Calculate the moist static energy of parcels.
@@ -1587,6 +1795,7 @@ def moist_static_energy(heights, temperature, specific_humidity):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]')
 def thickness_hydrostatic(pressure, temperature, **kwargs):
     r"""Calculate the thickness of a layer via the hypsometric equation.
@@ -1659,6 +1868,7 @@ def thickness_hydrostatic(pressure, temperature, **kwargs):
 
 
 @exporter.export
+@preprocess_xarray
 @check_units('[pressure]', '[temperature]')
 def thickness_hydrostatic_from_relative_humidity(pressure, temperature, relative_humidity,
                                                  **kwargs):
@@ -1710,3 +1920,238 @@ def thickness_hydrostatic_from_relative_humidity(pressure, temperature, relative
 
     return thickness_hydrostatic(pressure, temperature, mixing=mixing, bottom=bottom,
                                  depth=depth)
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[length]', '[temperature]')
+def brunt_vaisala_frequency_squared(heights, potential_temperature, axis=0):
+    r"""Calculate the square of the Brunt-Vaisala frequency.
+
+    Brunt-Vaisala frequency squared (a measure of atmospheric stability) is given by the
+    formula:
+
+    .. math:: N^2 = \frac{g}{\theta} \frac{d\theta}{dz}
+
+    This formula is based off of Equations 3.75 and 3.77 in [Hobbs2006]_.
+
+    Parameters
+    ----------
+    heights : array-like
+        One-dimensional profile of atmospheric height
+    potential_temperature : array-like
+        Atmospheric potential temperature
+    axis : int, optional
+        The axis corresponding to vertical in the potential temperature array, defaults to 0.
+
+    Returns
+    -------
+    array-like
+        The square of the Brunt-Vaisala frequency.
+
+    See Also
+    --------
+    brunt_vaisala_frequency, brunt_vaisala_period, potential_temperature
+
+    """
+    # Ensure validity of temperature units
+    potential_temperature = potential_temperature.to('K')
+
+    # Calculate and return the square of Brunt-Vaisala frequency
+    return g / potential_temperature * first_derivative(potential_temperature, x=heights,
+                                                        axis=axis)
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[length]', '[temperature]')
+def brunt_vaisala_frequency(heights, potential_temperature, axis=0):
+    r"""Calculate the Brunt-Vaisala frequency.
+
+    This function will calculate the Brunt-Vaisala frequency as follows:
+
+    .. math:: N = \left( \frac{g}{\theta} \frac{d\theta}{dz} \right)^\frac{1}{2}
+
+    This formula based off of Equations 3.75 and 3.77 in [Hobbs2006]_.
+
+    This function is a wrapper for `brunt_vaisala_frequency_squared` that filters out negative
+    (unstable) quanties and takes the square root.
+
+    Parameters
+    ----------
+    heights : array-like
+        One-dimensional profile of atmospheric height
+    potential_temperature : array-like
+        Atmospheric potential temperature
+    axis : int, optional
+        The axis corresponding to vertical in the potential temperature array, defaults to 0.
+
+    Returns
+    -------
+    array-like
+        Brunt-Vaisala frequency.
+
+    See Also
+    --------
+    brunt_vaisala_frequency_squared, brunt_vaisala_period, potential_temperature
+
+    """
+    bv_freq_squared = brunt_vaisala_frequency_squared(heights, potential_temperature,
+                                                      axis=axis)
+    bv_freq_squared[bv_freq_squared.magnitude < 0] = np.nan
+
+    return np.sqrt(bv_freq_squared)
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[length]', '[temperature]')
+def brunt_vaisala_period(heights, potential_temperature, axis=0):
+    r"""Calculate the Brunt-Vaisala period.
+
+    This function is a helper function for `brunt_vaisala_frequency` that calculates the
+    period of oscilation as in Exercise 3.13 of [Hobbs2006]_:
+
+    .. math:: \tau = \frac{2\pi}{N}
+
+    Returns `NaN` when :math:`N^2 > 0`.
+
+    Parameters
+    ----------
+    heights : array-like
+        One-dimensional profile of atmospheric height
+    potential_temperature : array-like
+        Atmospheric potential temperature
+    axis : int, optional
+        The axis corresponding to vertical in the potential temperature array, defaults to 0.
+
+    Returns
+    -------
+    array-like
+        Brunt-Vaisala period.
+
+    See Also
+    --------
+    brunt_vaisala_frequency, brunt_vaisala_frequency_squared, potential_temperature
+
+    """
+    bv_freq_squared = brunt_vaisala_frequency_squared(heights, potential_temperature,
+                                                      axis=axis)
+    bv_freq_squared[bv_freq_squared.magnitude <= 0] = np.nan
+
+    return 2 * np.pi / np.sqrt(bv_freq_squared)
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[pressure]', '[temperature]', '[temperature]')
+def wet_bulb_temperature(pressure, temperature, dewpoint):
+    """Calculate the wet-bulb temperature using Normand's rule.
+
+    This function calculates the wet-bulb temperature using the Normand method. The LCL is
+    computed, and that parcel brought down to the starting pressure along a moist adiabat.
+    Norman method (and others) described and compared by [Knox2017]_.
+
+    Parameters
+    ----------
+    pressure : `pint.Quantity`
+        Initial atmospheric pressure
+    temperature : `pint.Quantity`
+        Initial atmospheric temperature
+    dewpoint : `pint.Quantity`
+        Initial atmospheric dewpoint
+
+    Returns
+    -------
+    array-like
+        Wet-bulb temperature
+
+    See Also
+    --------
+    lcl, moist_lapse
+
+    """
+    if not hasattr(pressure, 'shape'):
+        pressure = atleast_1d(pressure)
+        temperature = atleast_1d(temperature)
+        dewpoint = atleast_1d(dewpoint)
+
+    it = np.nditer([pressure, temperature, dewpoint, None],
+                   op_dtypes=['float', 'float', 'float', 'float'],
+                   flags=['buffered'])
+
+    for press, temp, dewp, ret in it:
+        press = press * pressure.units
+        temp = temp * temperature.units
+        dewp = dewp * dewpoint.units
+        lcl_pressure, lcl_temperature = lcl(press, temp, dewp)
+        moist_adiabat_temperatures = moist_lapse(concatenate([lcl_pressure, press]),
+                                                 lcl_temperature)
+        ret[...] = moist_adiabat_temperatures[-1]
+
+    # If we started with a scalar, return a scalar
+    if it.operands[3].size == 1:
+        return it.operands[3][0] * moist_adiabat_temperatures.units
+    return it.operands[3] * moist_adiabat_temperatures.units
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[pressure]', '[temperature]')
+def static_stability(pressure, temperature, axis=0):
+    r"""Calculate the static stability within a vertical profile.
+
+    .. math:: \sigma = -\frac{RT}{p} \frac{\partial \ln \theta}{\partial p}
+
+    This formuala is based on equation 4.3.6 in [Bluestein1992]_.
+
+    Parameters
+    ----------
+    pressure : array-like
+        Profile of atmospheric pressure
+    temperature : array-like
+        Profile of temperature
+    axis : int, optional
+        The axis corresponding to vertical in the pressure and temperature arrays, defaults
+        to 0.
+
+    Returns
+    -------
+    array-like
+        The profile of static stability.
+
+    """
+    theta = potential_temperature(pressure, temperature)
+
+    return - Rd * temperature / pressure * first_derivative(np.log(theta / units.K),
+                                                            x=pressure, axis=axis)
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[dimensionless]', '[temperature]', '[pressure]')
+def dewpoint_from_specific_humidity(specific_humidity, temperature, pressure):
+    r"""Calculate the dewpoint from specific humidity, temperature, and pressure.
+
+    Parameters
+    ----------
+    specific_humidity: `pint.Quantity`
+        Specific humidity of air
+    temperature: `pint.Quantity`
+        Air temperature
+    pressure: `pint.Quantity`
+        Total atmospheric pressure
+
+    Returns
+    -------
+    `pint.Quantity`
+        Dewpoint temperature
+
+    See Also
+    --------
+    relative_humidity_from_mixing_ratio, dewpoint_rh
+
+    """
+    return dewpoint_rh(temperature, relative_humidity_from_specific_humidity(specific_humidity,
+                                                                             temperature,
+                                                                             pressure))
