@@ -1,4 +1,4 @@
-# Copyright (c) 2015,2017 MetPy Developers.
+# Copyright (c) 2015,2017,2018 MetPy Developers.
 # Distributed under the terms of the BSD 3-Clause License.
 # SPDX-License-Identifier: BSD-3-Clause
 """Utilities for use in making plots."""
@@ -15,10 +15,11 @@ import pkg_resources
 from ..units import concatenate
 
 
-def add_timestamp(ax, time=None, x=0.99, y=-0.04, ha='right', high_contrast=False, **kwargs):
-    """Add a timestamp at plot creation time.
+def add_timestamp(ax, time=None, x=0.99, y=-0.04, ha='right', high_contrast=False,
+                  pretext='Created: ', time_format='%Y-%m-%dT%H:%M:%SZ', **kwargs):
+    """Add a timestamp to a plot.
 
-    Adds an ISO format timestamp with the time of plot creation to the plot.
+    Adds a timestamp to a plot, defaulting to the time of plot creation in ISO format.
 
     Parameters
     ----------
@@ -34,6 +35,10 @@ def add_timestamp(ax, time=None, x=0.99, y=-0.04, ha='right', high_contrast=Fals
         Horizontal alignment of the time stamp string
     high_contrast : bool
         Outline text for increased contrast
+    pretext : str
+        Text to appear before the timestamp, optional. Defaults to 'Created: '
+    time_format : str
+        Display format of time, optional. Defaults to ISO format.
 
     Returns
     -------
@@ -50,7 +55,7 @@ def add_timestamp(ax, time=None, x=0.99, y=-0.04, ha='right', high_contrast=Fals
     text_args.update(**kwargs)
     if not time:
         time = datetime.utcnow()
-    timestr = datetime.strftime(time, 'Created: %Y-%m-%dT%H:%M:%SZ')
+    timestr = pretext + time.strftime(time_format)
     return ax.text(x, y, timestr, ha=ha, transform=ax.transAxes, **text_args)
 
 
@@ -116,8 +121,8 @@ def add_metpy_logo(fig, x=10, y=25, zorder=100, size='small', **kwargs):
 
     Returns
     -------
-    fig : `matplotlib.figure`
-       The `figure` instance used for plotting
+    `matplotlib.image.FigureImage`
+       The `matplotlib.image.FigureImage` instance created
 
     """
     return _add_logo(fig, x=x, y=y, zorder=zorder, which='metpy', size=size, **kwargs)
@@ -144,8 +149,8 @@ def add_unidata_logo(fig, x=10, y=25, zorder=100, size='small', **kwargs):
 
     Returns
     -------
-    fig : `matplotlib.figure`
-       The `figure` instance used for plotting
+    `matplotlib.image.FigureImage`
+       The `matplotlib.image.FigureImage` instance created
 
     """
     return _add_logo(fig, x=x, y=y, zorder=zorder, which='unidata', size=size, **kwargs)
