@@ -20,8 +20,8 @@ import matplotlib.transforms as transforms
 import numpy as np
 
 from ._util import colored_line
-from ..calc import dewpoint, dry_lapse, moist_lapse, vapor_pressure
-from ..calc.tools import delete_masked_points, interp
+from ..calc import dewpoint, dry_lapse, interp, moist_lapse, vapor_pressure
+from ..calc.tools import _delete_masked_points
 from ..package_tools import Exporter
 from ..units import concatenate, units
 
@@ -314,7 +314,7 @@ class SkewT(object):
 
         """
         # Skew-T logP plotting
-        t, p = delete_masked_points(t, p)
+        t, p = _delete_masked_points(t, p)
         lines = self.ax.semilogy(t, p, *args, **kwargs)
 
         # Disables the log-formatting that comes with semilogy
@@ -356,6 +356,8 @@ class SkewT(object):
         y_clip_radius : float, optional
             Space, in normalized axes coordinates, to leave above/below plot
             before clipping wind barbs in the y-direction. Defaults to 0.08.
+        plot_units: `pint.unit`
+            Units to plot in (performing conversion if necessary). Defaults to given units.
         kwargs
             Other keyword arguments to pass to :func:`~matplotlib.pyplot.barbs`
 
@@ -369,6 +371,16 @@ class SkewT(object):
         :func:`matplotlib.pyplot.barbs`
 
         """
+        # If plot_units specified, convert the data to those units
+        plotting_units = kwargs.pop('plot_units', None)
+        if plotting_units:
+            if hasattr(u, 'units') and hasattr(v, 'units'):
+                u = u.to(plotting_units)
+                v = v.to(plotting_units)
+            else:
+                raise ValueError('To convert to plotting units, units must be attached to '
+                                 'u and v wind components.')
+
         # Assemble array of x-locations in axes space
         x = np.empty_like(p)
         x.fill(xloc)
@@ -594,7 +606,7 @@ class SkewT(object):
         if matplotlib.__version__ >= '2.1':
             fill_args['interpolate'] = True
 
-        arrs = delete_masked_points(*arrs)
+        arrs = _delete_masked_points(*arrs)
 
         return self.ax.fill_betweenx(*arrs, **fill_args)
 
@@ -771,9 +783,35 @@ class Hodograph(object):
 
         """
         line_args = self._form_line_args(kwargs)
-        u, v = delete_masked_points(u, v)
+        u, v = _delete_masked_points(u, v)
         return self.ax.plot(u, v, **line_args)
 
+    def wind_vectors(self, u, v, **kwargs):
+        r"""Plot u, v data as wind vectors.
+
+        Plot the wind data as vectors for each level, beginning at the origin.
+
+        Parameters
+        ----------
+        u : array_like
+            u-component of wind
+        v : array_like
+            v-component of wind
+        kwargs
+            Other keyword arguments to pass to :meth:`matplotlib.axes.Axes.quiver`
+
+        Returns
+        -------
+        matplotlib.quiver.Quiver
+            arrows plotted
+
+        """
+        quiver_args = {'units': 'xy', 'scale': 1}
+        quiver_args.update(**kwargs)
+        center_position = np.zeros_like(u)
+        return self.ax.quiver(center_position, center_position,
+                              u, v, **quiver_args)
+
     def plot_colormapped(self, u, v, c, bounds=None, colors=None, **kwargs):
         r"""Plot u, v data, with line colored based on a third set of data.
 
@@ -814,18 +852,33 @@ class Hodograph(object):
         :meth:`Hodograph.plot`
 
         """
-        u, v, c = delete_masked_points(u, v, c)
+        u, v, c = _delete_masked_points(u, v, c)
 
         # Plotting a color segmented hodograph
         if colors:
             cmap = mcolors.ListedColormap(colors)
             # If we are segmenting by height (a length), interpolate the bounds
             if bounds.dimensionality == {'[length]': 1.0}:
-                bounds = bounds + c[0]  # Make all heights AGL
-                interpolation_heights = concatenate((bounds, c))
+
+                # Find any bounds not in the data and interpolate them
+                interpolation_heights = [bound.m for bound in bounds if bound not in c]
+                interpolation_heights = np.array(interpolation_heights) * bounds.units
                 interpolation_heights = (np.sort(interpolation_heights) *
                                          interpolation_heights.units)
-                c, u, v = interp(interpolation_heights, c, c, u, v)
+                (interpolated_heights, interpolated_u,
+                 interpolated_v) = interp(interpolation_heights, c, c, u, v)
+
+                # Combine the interpolated data with the actual data
+                c = concatenate([c, interpolated_heights])
+                u = concatenate([u, interpolated_u])
+                v = concatenate([v, interpolated_v])
+                sort_inds = np.argsort(c)
+                c = c[sort_inds]
+                u = u[sort_inds]
+                v = v[sort_inds]
+
+                # Unit conversion required for coloring of bounds/data in dissimilar units
+                # to work properly.
                 c = c.to_base_units()  # TODO: This shouldn't be required!
                 bounds = bounds.to_base_units()
             # If segmenting by anything else, do not interpolate, just use the data
