@@ -32,6 +32,41 @@ units.define(pint.unit.UnitDefinition('percent', '%', (),
              pint.converters.ScaleConverter(0.01)))
 
 
+def pandas_dataframe_to_unit_arrays(df, column_units=None):
+    """Attach units to data in pandas dataframes and return united arrays.
+
+    Parameters
+    ----------
+    df : `pandas.DataFrame`
+        Data in pandas dataframe.
+
+    column_units : dict
+        Dictionary of units to attach to columns of the dataframe. Overrides
+        the units attribute if it is attached to the dataframe.
+
+    Returns
+    -------
+        Dictionary containing united arrays with keys corresponding to the dataframe
+        column names.
+
+    """
+    if not column_units:
+        try:
+            column_units = df.units
+        except AttributeError:
+            raise ValueError('No units attribute attached to pandas '
+                             'dataframe and col_units not given.')
+
+    # Iterate through columns attaching units if we have them, if not, don't touch it
+    res = {}
+    for column in df:
+        if column in column_units and column_units[column]:
+            res[column] = df[column].values * units(column_units[column])
+        else:
+            res[column] = df[column].values
+    return res
+
+
 def concatenate(arrs, axis=0):
     r"""Concatenate multiple values into a new unitized object.
 
@@ -66,7 +101,13 @@ def concatenate(arrs, axis=0):
             a = a.to(dest).magnitude
         data.append(np.atleast_1d(a))
 
-    return units.Quantity(np.concatenate(data, axis=axis), dest)
+    # Use masked array concatenate to ensure masks are preserved, but convert to an
+    # array if there are no masked values.
+    data = np.ma.concatenate(data, axis=axis)
+    if not np.any(data.mask):
+        data = np.asarray(data)
+
+    return units.Quantity(data, dest)
 
 
 def diff(x, **kwargs):
@@ -304,7 +345,10 @@ except (AttributeError, RuntimeError):  # Pint's not available, try to enable ou
         @staticmethod
         def default_units(x, axis):
             """Get the default unit to use for the given combination of unit and axis."""
-            return getattr(x, 'units', None)
+            if isinstance(x, (tuple, list)):
+                return getattr(x[0], 'units', 'dimensionless')
+            else:
+                return getattr(x, 'units', 'dimensionless')
 
     # Register the class
     munits.registry[units.Quantity] = PintConverter(units)
